<?php
/*
	Version: 9
	Version date: 11.05.2009
*/
class Cuscus {
	/**
	 * If === false, then the connection has not been established
	 * Else, it is a resource and the connection has been established
	 *
	 * @var unknown_type
	 */
	var $link_identifier;
	/**
	 * If === false, then the notification function will not be called
	 * else, the notification function with the name $notification_function_name will be called. It is assumed that it is of type string.
	 *
	 * @var unknown_type
	 */
	var $notification_function_name;
	var $slow_query_time;
	var $slow_query_notification_function;
	var $log_query = false;
	var $log_table = '';
	function Cuscus() {
		$this->link_identifier = false;
		$this->notification_function_name = false;
		$this->slow_query_time = -1;
		$this->slow_query_notification_function = false;
	}
	function call_notification_function($error) {
		if ($this->notification_function_name !== false) {
			call_user_func($this->notification_function_name, $error);
		}
	}
	function call_slow_query_notification_function($data) {
		if ($this->slow_query_notification_function !== false) {
			call_user_func($this->slow_query_notification_function, $data);
		}
	}
	/**
	 * connects to the MySQL server with the given parameters
	 *
	 * @param string $server
	 * @param string $username
	 * @param string $password
	 * @param bool $new_link
	 * @param int $client_flags
	 * @return [object of type Cuscus_Error] or [true - when the connection succeeds]
	 */
	function connect($server = null, $username = null, $password = null, $new_link = null, $client_flags = null) {
		$link_identifier = @mysql_connect($server, $username, $password, $new_link, $client_flags);
		if ($link_identifier === false) {
			$error = $this->compile_error_info('mysql_connect() returned false', 'connect()', 1, array('server' => $server, 'username' => $username, 'password' => $password, 'new_link' => $new_link, 'client_flags' => $client_flags));
			$this->call_notification_function($error);
			return $error;
		}
		else {
			//a resource was returned
			$this->link_identifier = $link_identifier;
			return true;
		}
	}
	function is_error(&$object) {
		return $object instanceof Cuscus_Error;
	}
	function no_rows(&$object) {
		return $object instanceof Cuscus_NO_ROWS;
	}
	function compile_error_info($reason, $function, $code, $function_arguments) {
		$data = array();
		$data['mysql_error'] = @mysql_error();
		$data['mysql_errno'] = @mysql_errno();
		$data['reason'] = $reason;
		$data['function'] = $function;
		$data['code'] = $code;
		$data['function_arguments'] = $function_arguments;
		$Cuscus_Error_object = new Cuscus_Error($data);
		return $Cuscus_Error_object;
	}
	function select_database($new_database) {
		$ret = @mysql_select_db($new_database, $this->link_identifier);
		if ($ret === false) {
			//an error ocurred
			$error = $this->compile_error_info('mysql_select_db() returned false', 'select_database()', 2, array('new_database' => $new_database));
			$this->call_notification_function($error);
			return $error;
		}
		else {
			//true. connection succeeded
			return true;
		}
	}
	function q($query) {
		$start_time = $this->microtime_double();
		$ret = @mysql_query($query, $this->link_identifier);
		$end_time = $this->microtime_double();
		if (($end_time - $start_time) >= $this->slow_query_time && $this->slow_query_time !== -1) {
			//the query was slow
			$data['query'] = $query;
			$data['time'] = $end_time - $start_time;
			$this->call_slow_query_notification_function($data);
		}
		if ($this->log_query === true) {
			@mysql_query('INSERT INTO ' . $this->log_table . ' SET query_text = "' . addslashes($query) . '", occur_time = "' . date('Y-m-d H:i:s') . '"');
		}
		if ($ret === false) {
			//error
			$error = $this->compile_error_info('mysql_query returned false', 'q()', 3, array('query' => $query));
			$this->call_notification_function($error);
			return $error;
		}
		else {
			//the query was successful
			return $ret;
		}
	}
	function s($select_what, $table = '', $postconditions = '') {
		$query = 'SELECT '. $select_what;
		if ($table !== '') {
			$query .= ' FROM '. $table;
		}
		if ($postconditions !== '') {
			$query .= ' '. $postconditions;
		}
		$ret = $this->q($query);
		return $ret;
	}
	function sfrfv($select_what, $table = '', $postconditions = '') {
		$ret = $this->s($select_what, $table, $postconditions);
		if (Cuscus::is_error($ret)) return $ret;
		//there was no error
		$rows = @mysql_num_rows($ret);
		if ($rows === 0) return new Cuscus_NO_ROWS();
		else {
			$row = @mysql_fetch_row($ret);
			return $row[0];
		}
	}
	function sfrfvi($select_what, $table = '', $postconditions = '') {
		$ret = $this->sfrfv($select_what, $table, $postconditions);
		if (Cuscus::is_error($ret)) return $ret;
		if (Cuscus::no_rows($ret)) return $ret;
		//we received a valid value
		return intval($ret);
	}
	function sfrfvd($select_what, $table = '', $postconditions = '') {
		$ret = $this->sfrfv($select_what, $table, $postconditions);
		if (Cuscus::is_error($ret)) return $ret;
		if (Cuscus::no_rows($ret)) return $ret;
		//we received a valid value
		return doubleval($ret);
	}
	function staa($select_what, $table = '', $postconditions = '') {
		$ret = $this->s($select_what, $table, $postconditions);
		if (Cuscus::is_error($ret)) return $ret;
		if (@mysql_num_rows($ret) === 0) return array();
		//we have a valid row
		$data = @mysql_fetch_assoc($ret);
		return $data;
	}
	function staas($select_what, $table = '', $postconditions = '') {
		$ret = $this->s($select_what, $table, $postconditions);
		if (Cuscus::is_error($ret)) return $ret;
		if (@mysql_num_rows($ret) === 0) return array();
		//we have some rows
		$result = array();
		while ($row = @mysql_fetch_assoc($ret)) {
			$result[] = $row;
		}
		return $result;
	}
	/**
	 * Inserts one row of data into $table.
	 *
	 * @param string $table
	 * @param string $data
	 */
	function i($table, $data) {
		$query = 'INSERT INTO '. $table . ' SET ';
		$col_count = 0;
		foreach ($data as $col => $val) {
			if ($col_count > 0) $query .= ', ';
			if (is_integer($col)) {
				//we do not determine the variable type, the user took care of it all.
				$query .= $val;
			}
			elseif (is_string($val)) $query .= $col .' = "'. $val .'"';
			else $query .= $col .' = '. $val;
			++$col_count;
		}
		return $this->q($query);
	}
	function u($table, $data, $postconditions = '') {
		$query = 'UPDATE '. $table . ' SET ';
		$col_count = 0;
		foreach ($data as $col => $val) {
			if ($col_count > 0) $query .= ', ';
			if (is_integer($col)) {
				//we do not determine the variable type, the user took care of it all.
				$query .= $val;
			}
			elseif (is_string($val)) $query .= $col .' = "'. $val .'"';
			else $query .= $col .' = '. $val;
			++$col_count;
		}
		$query .= ($postconditions === '' ? '' : ' '. $postconditions);
		return $this->q($query);
	}
	function d($table, $postconditions = '') {
		$query = 'DELETE FROM '. $table .($postconditions === '' ? '' : ' '. $postconditions);
		return $this->q($query);
	}
	function microtime_double() {
    	list($usec, $sec) = explode(" ", microtime());
    	return (double)((double)$usec + (integer)$sec);
	}
	function sd($new_database) {
		$result = @mysql_select_db($new_database, $this->link_identifier);
		if ($result === false) {
			//error
			$error = $this->compile_error_info('mysql_select_db returned false', 'sd()', 4, array('new_database' => $new_database));
			$this->call_notification_function($error);
			return $error;
		}
		else {
			return true;
		}
	}
	/**
	 * Real Escape string
	 *
	 * @param string $str
	 * @return false/string
	 */
	function re($str) {
		//escaped string
		$es = @mysql_real_escape_string($str, $this->link_identifier);
		if ($es === false) {
			$error = $this->compile_error_info('mysql_real_escape_string returned false', 're()', 5, array('str' => $str));
			$this->call_notification_function($error);
			return $error;
		}
		return $es;
	}
	/**
	 * Get Affected Rows
	 *
	 * @return int
	 */
	function gar() {
		return  mysql_affected_rows($this->link_identifier);
	}
	/**
	 * Last inserted ID
	 *
	 * @return unknown
	 */
	function lid() {
		return $this->sfrfv('LAST_INSERT_ID()');
	}
	/**
	 * Returns FOUND_ROWS() - presumes SQL_CAL_FOUND_ROWS
	 *
	 * @return str
	 */
	function fr() {
		return $this->sfrfv('FOUND_ROWS()');
	}
	/**
	 * count
	 *
	 * @param str $table
	 * @param str $postconditions
	 * @return unknown
	 */
	function c($table, $postconditions) {
		return $this->sfrfvi('count(*)', $table, $postconditions);
	}
	/**
	 * No Array - false when array and contains elements
	 *
	 * @param unknown_type $res
	 * @return bool
	 */
	function na($res) {
		return !is_array($res) || (is_array($res) && count($res) === 0);
	}
}
?>