<?php
define('SENDING_THE_EMAIL_FAILED', -1);

class MailQueue {
	/**
	 * This variable is initialized by the constuctor.
	 *
	 * @var Griffin object
	 */
	var $griffin;
	/**
	 * This is the name of the MySQL table where the mail queue is kept. You can set it manually, but there is a function for your convenience, called SetMQTable();
	 *
	 * @var string
	 */
	var $mq_table;
	/**
	 * the name of the table where the configuration variables AND stats are.
	 *
	 * @var string
	 */
	var $config_table;
	function MailQueue() {
		$this->griffin = new Griffin();
		return true;
	}
	/**
	 * This function connects the griffin with the given parameters to the MySQL server, where the mail queue is (inside tables ofcourse ]).
	 * You can also manually connect griffin by accessing $this->griffin->..., because this function is only provided for your convenience
	 *
	 * @param string $username
	 * @param string $hostname
	 * @param string $password
	 * @param string $database
	 * @return the value of $this->griffin->Connect($password);
	 */
	function Init_Griffin($username, $hostname, $password, $database) {
		$this->griffin->SetUsername($username);
		$this->griffin->SetHostname($hostname);
		$connect_result = $this->griffin->Connect($password);
		$this->griffin->SelectDatabase($database);
		return $connect_result;
	}
	/**
	 * Sets the mail queue table's name
	 *
	 * @param string $mail_queue_table_name
	 * @return true (always)
	 */
	function SetMQTable($mail_queue_table_name) {
		$this->mq_table = $mail_queue_table_name;
		return true;
	}
	function SetConfigTable($config_table_name) {
		$this->config_table = $config_table_name;
		return true;
	}
	/**
	 * Inserts an email into the queue.
	 *
	 * @param string $to
	 * @param string $subject
	 * @param string $message
	 * @param string $additional_headers
	 * @param string $additional_parameters
	 * @return the result of the 'i' member function of $this->griffin
	 */
	function InsertEmail($to, $subject, $message, $additional_headers = '', $additional_parameters = '') {
		return $this->griffin->i(array('to_email' => addslashes($to), 'subject' => addslashes($subject), 'message' => addslashes($message), 'additional_headers' => addslashes($additional_headers), 'additional_parameters' => addslashes($additional_parameters), 'add_time = now()'), $this->mq_table);
	}
	/**
	 * Updates the time that the emails are held after the next portion has been sent
	 *
	 * @param integer $time_in_seconds
	 * @return result of $this->griffin->u
	 */
	function UpdateHoldTime($time_in_seconds) {
		return $this->griffin->u(array('hold_time' => intval($time_in_seconds)), $this->config_table);
	}
	/**
	 * Updates the number of emails that are sent each turn
	 *
	 * @param integer $number_of_emails
	 * @return result of $this->griffin->u
	 */
	function UpdateEmailsPerTurn($number_of_emails) {
		return $this->griffin->u(array('emails_per_turn' => intval($number_of_emails)), $this->config_table);
	}
	function UpdateTotalEmailsSent($number_of_emails) {
		return $this->griffin->u(array('total_emails_sent' => intval($number_of_emails)), $this->config_table);
	}
	function UpdateSentToday($number_of_emails) {
		//if "it's already tomorrow", then we nullify the column
		if ($this->GetToday() !== date('Y-m-d')) {
			$this->griffin->u(array('today' => date('Y-m-d')), $this->config_table);
			return $this->griffin->u(array('sent_today' => intval($number_of_emails)), $this->config_table);
		}
		else {
			return $this->griffin->u(array('sent_today = sent_today + '. intval($number_of_emails)), $this->config_table);
		}
	}
	function UpdateLastSend($new_time) {
		return $this->griffin->u(array('last_send' => $new_time), $this->config_table);
	}
	function GetLastSend() {
		return $this->griffin->sfrfv('last_send', $this->config_table);
	}
	function GetTotalEmailsSent() {
		return $this->griffin->sfrfv('total_emails_sent', $this->config_table);
	}
	function GetSentToday() {
		return $this->griffin->sfrfv('sent_today', $this->config_table);
	}
	function GetToday() {
		return $this->griffin->sfrfv('today', $this->config_table);
	}
	function GetHoldTime() {
		return $this->griffin->sfrfv('hold_time', $this->config_table);
	}
	function GetEmailsPerTurn() {
		return $this->griffin->sfrfv('emails_per_turn', $this->config_table);
	}
	function Send() {
		$last_send = strtotime($this->GetLastSend());
		$hold_time = intval($this->GetHoldTime());
		if (time() - $hold_time >= $last_send) {
			//send the emails
			$emails_per_turn = $this->GetEmailsPerTurn();
			$emails_to_send = $this->griffin->staas('*', $this->mq_table, 'ORDER BY ID ASC LIMIT '. $emails_per_turn);
			$emails_sent = 0;
			foreach ($emails_to_send as $num => $data) {
				if (mail($data['to_email'], $data['subject'], $data['message'], $data['additional_headers'], $data['additional_parameters'])) {
					++$emails_sent;
					$this->DeleteEmail($data['id']);
				}
				else {
					//sending the email failed
					$this->UpdateLastSend($this->CorrectDate(time()));
					return SENDING_THE_EMAIL_FAILED;
				}
			}
			//all emails were sent successfully
			$this->UpdateTotalEmailsSent($this->GetTotalEmailsSent() + $emails_sent);
			$this->UpdateSentToday($emails_sent);
			$this->UpdateLastSend($this->CorrectDate(time()));
			return true;
		}
	}
	function DeleteEmail($email_id) {
		$this->griffin->d($this->mq_table, 'WHERE id = '. $email_id);
	}
	function CorrectDate($current_timestamp) {
		return date('Y-m-d G:i:s', $current_timestamp);
	}
}
?>